<?php

namespace Opencart\System\Library\Extension\Chatbro;

use Opencart\System\Library\Extension\Chatbro\Library;
use Opencart\System\Library\Extension\Chatbro\StoreloaderTrait;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Shortcodes\CbroShortCode;

use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroUserBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Common\CbroCommon;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Users\CbroUser;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Admin\CbroAdmin;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Utils\CbroUtils;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Settings\CbroSettings;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Chats\CbroChats;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Api\CbroApi;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Chats\CbroDisplayChat;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Permissions\CbroPermissions;

use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroUtilsBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroAdminBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroSettingsBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroChatsBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroApiBackend;
use Opencart\System\Library\Extension\Chatbro\Chatbroapp\Backends\CbroPermissionsBackend;

class Chatbrochats extends Library
{
    use StoreloaderTrait;

    const DEBUG_LOG_FILENAME = 'chatbro.%s.log';

    public $store_url;
    public $store_name;
    public $store_id = 0;
    private $debug_log;

    public function __construct($registry, $store_id)
    {
        parent::__construct($registry);

        $this->store_id = $store_id;

        $this->load->model('setting/setting');

        if ($this->store_id === 0) {
            $this->store_url = defined('HTTP_CATALOG') ? HTTP_CATALOG : HTTP_SERVER;
            $this->store_name = $this->config->get('config_name');
        } else {
            $this->store_url = $this->model_setting_setting->getValue('config_url', $store_id);
            $this->store_name = $this->model_setting_setting->getValue('config_name', $store_id);
        }

        $this->loadStore($this->store_id);

        $this->debug_log = new Log(sprintf(self::DEBUG_LOG_FILENAME, $this->store_id));
    }

    public function getStoreUrl()
    {
        return $this->store_url;
    }

    public function admin()
    {
        return CbroAdmin::display();
    }

    public function getChats()
    {
        return CbroAdmin::get_chats();
    }

    public function deleteChat()
    {
        return CbroAdmin::delete_chat();
    }

    public function updateChat()
    {
        return CbroAdmin::update_chat();
    }

    public function createChat()
    {
        return CbroAdmin::create_chat();
    }

    public function saveSettings()
    {
        return CbroAdmin::save_settings();
    }

    public function getShortcodeInstance()
    {
        return CBroShortCode::get_instance();
    }

    public function displayChats()
    {
        $result = "";

        foreach (CbroChats::iterator() as $name => $chat) {
            $result .= (new CbroDisplayChat($chat))->get_sitewide_popup_chat_code();
        }

        return $result;
    }

    public function init()
    {
        CbroUtils::init(new CbroUtilsBackend($this));
        CbroCommon::init($this);
        CbroAdmin::init(new CbroAdminBackend());
        CbroSettings::init(new CbroSettingsBackend());
        CbroChats::init(new CbroChatsBackend());
        CbroApi::init(new CbroApiBackend());
        CbroUser::init(new CbroUserBackend($this));
        CbroPermissions::init(new CbroPermissionsBackend());

        if (!class_exists("CBroShortCode")) {
            CbroShortCode::get_instance();
        }

        CbroChats::init_chats();
    }

    public function debugLog($text)
    {
        $this->debug_log->write($text);
    }
}
